# Reference

<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

### Resource types

* [`debconf`](#debconf): Manage debconf database entries on Debian based systems.

## Resource types

### <a name="debconf"></a>`debconf`

This type can either set or remove a value for a debconf database
entry. It uses multiple programs from the `debconf` package.

#### Examples

##### Set a select value

```puppet
debconf { 'tzdata/Areas':
  type  => 'select',
  value => 'Europe',
}
```

##### Set a boolean value

```puppet
debconf { 'dash/sh':
  type  => 'boolean',
  value => 'true',
}
```

##### Set a boolean value in a specified package and mark as seen

```puppet
debconf { 'libraries/restart-without-asking':
  package => 'libc6',
  type    => 'boolean',
  value   => 'true',
  seen    => true,
}
```

#### Properties

The following properties are available in the `debconf` type.

##### `ensure`

Valid values: `present`, `absent`

Specifies whether the resource should exist. Setting this to
"absent" tells Puppet to remove the debconf entry if it exists, and
negates the effect of any other parameters.

Default value: `present`

##### `seen`

Valid values: ``true``, ``false``

The value of the 'seen' flag. This can be left undefined or be one
of the boolean values true or false.

##### `value`

Valid values: `%r{\S}`

The value for the item (e.g. 'Europe').

#### Parameters

The following parameters are available in the `debconf` type.

* [`item`](#item)
* [`name`](#name)
* [`package`](#package)
* [`provider`](#provider)
* [`type`](#type)

##### <a name="item"></a>`item`

Valid values: `%r{^[a-z0-9][a-z0-9:.+-]+\/[a-zA-Z0-9\/_.+-]+$}`

The item name. This string must have the following format: the
package name, a literal slash char and the name of the question (e.g.
'tzdata/Areas'). The default value is the title of the resource.

##### <a name="name"></a>`name`

namevar

The name of the resource. If the parameter 'item' is not set, then
this value will be used for it. You can set the same item in different
packages by using different names for the resources.

##### <a name="package"></a>`package`

Valid values: `%r{^[a-z0-9][a-z0-9:.+-]+$}`

The package the item belongs to. The default is the first part (up
to the first '/') of the item parameter (e.g. 'tzdata').

##### <a name="provider"></a>`provider`

The specific backend to use for this `debconf` resource. You will seldom need to specify this --- Puppet will usually
discover the appropriate provider for your platform.

##### <a name="type"></a>`type`

Valid values: `string`, `boolean`, `select`, `multiselect`, `note`, `text`, `password`, `title`

The type of the item. This can only be one of the following
values: string, boolean, select, multiselect, note, text, password,
title.

