# configure environment to install controller from .deb and dev repo
class space_controller(
  Hash[String,Hash] $packages    = {},
  Hash[String,Hash] $files       = {},
  Optional[Tuple]   $extra_facts = {},
) {

  create_resources(package, $packages)

  $veil_global_group = lookup('veil_global_group')
  $veil_controller_user = lookup('veil_controller_user')

  $file_defaults = {
     owner => $veil_controller_user,
     group => $veil_global_group,
     mode  => '0755',
  }

  create_resources(file, $files, $file_defaults)

  require space_utils

  space_facts::addfacter { $name:
    extra_facts => $extra_facts,
  }

  require space_controller_settings

  # TEST5
  file { "/var/lib/ecp-veil/tests/ctr_init_test5.sh":
    ensure  => present,
    content => template("${module_name}/ctr_init_test5.sh.erb"),
    mode    => '0550',
  }


  contain '::space_controller::nginx_slave_inc'
  contain '::space_controller::repo_params'
  contain '::space_controller::install'
  contain '::space_controller::install_deps'
  contain '::space_controller::init_services'
  contain '::space_controller::controller_conf'
  contain '::space_controller::linstor_conf'
  contain '::space_controller::dirs'
  contain '::space_controller::class_deps'
  contain '::space_controller::serial'
  contain '::space_controller::settings'
  contain '::space_controller::finalize'

  Class['space_controller::nginx_slave_inc']
  -> Class['space_controller::repo_params']
  -> Class['space_controller::install']
  -> Class['space_controller::install_deps']
  -> Class['space_controller::init_services']
  -> Class['space_controller::controller_conf']
  -> Class['space_controller::linstor_conf']
  -> Class['space_controller::dirs']
  -> Class['space_controller::class_deps']
  -> Class['space_controller::serial']
  -> Class['space_controller::settings']
  -> Class['space_controller::finalize']

  $crontab_entries = lookup('space_controller::regenerate_system_token::crontab', Hash[String,Hash], 'hash', {})
  create_resources (cron, $crontab_entries)

  if defined('$::controller_slave_ip') {
    if $controller_slave_ip != "127.0.0.1" {
      cron { 'space_controller_master_sync':
        user        => 'root',
        minute      => '*',
        command     => 'nice -n 19 ionice -c 3 /usr/bin/flock -w0 -x /tmp/pa.lock /usr/local/sbin/veil-controller -u > /var/log/space_controller_sync.log 2>&1',
        environment => 'PATH=/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin',
      }
    } else {
      cron { 'space_controller_master_sync':
        ensure      => absent,
        user        => 'root',
        minute      => '*',
        command     => 'nice -n 19 ionice -c 3 /usr/bin/flock -w0 -x /tmp/pa.lock /usr/local/sbin/veil-controller -u > /var/log/space_controller_sync.log 2>&1',
        environment => 'PATH=/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin',
      }
    }
  }

  if defined('$::controller_master_ip') {
    if $controller_master_ip != "127.0.0.1" {
      cron { 'space_controller_slave_sync':
        user        => 'root',
        minute      => '*',
        command     => 'nice -n 19 ionice -c 3 /usr/bin/flock -w0 -x /tmp/pa.lock /usr/local/sbin/veil-controller -c > /var/log/space_controller_sync.log 2>&1',
        environment => 'PATH=/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin',
      }
    } else {
      cron { 'space_controller_slave_sync':
        ensure      => absent,
        user        => 'root',
        minute      => '*',
        command     => 'nice -n 19 ionice -c 3 /usr/bin/flock -w0 -x /tmp/pa.lock /usr/local/sbin/veil-controller -c > /var/log/space_controller_sync.log 2>&1',
        environment => 'PATH=/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin',
      }
    }
  }
}
