# configure environment to install controller from .deb and dev repo
class space_log(
  Optional[Tuple]   $extra_facts = {},
  Hash[String,Tuple] $loops = {},
  Hash[String,Hash] $controller_logs = {},
  Hash[String,Hash] $node_logs = {},
  Hash[String,Hash] $common_logs = {},
  Hash[String,Hash] $defaults = {},
) {

  space_facts::addfacter { $name:
    extra_facts => $extra_facts,
  }

  contain '::space_log::rsyslog_severity'

  $veil_global_group = lookup('veil_global_group')
  $veil_controller_user = lookup('veil_controller_user')

  # clean old config files. If config delete or rename
  tidy { "/etc/td-agent/conf.d":
    path    => "/etc/td-agent/conf.d",
    recurse => true,
    matches => [ '*.conf' ],
    rmdirs  => false,
  }
  
  # for gem 
  ensure_resource('package', 'ruby', {'ensure' => 'present'})
  ensure_resource('package', 'ruby-dev', {'ensure' => 'present'})
  ensure_resource('package', 'gcc', {'ensure' => 'present'})
  ensure_resource('package', 'make', {'ensure' => 'present'})

  if ! defined('$controller_ip') {
    $controller_ip = hiera('controller_ip', '127.0.0.1')
  }

  $notify_defaults = {
    notify => Class['fluentd::service']
  }

  include fluentd

  fluentd::plugin { 'faraday':
     type   => 'file',
     source => "puppet:///modules/${module_name}/faraday-0.15.4.gem",
   }

   exec { 'grafana_loki_install':
    command => "td-agent-gem install -N --conservative --minimal-deps --no-document --force --local /etc/puppetveil/code/environments/production/modules/space_log/files/fluent-plugin-grafana-loki-1.2.20.gem",
    unless  => "td-agent-gem list --local | grep fluent-plugin-grafana-loki | grep 1.2.20",
  }

  fluentd::plugin { 'fluent-plugin-grafana-loki':
     type   => 'file',
     source => "puppet:///modules/${module_name}/fluent-plugin-grafana-loki-1.2.20.gem",
   }

  if defined('$is_controller_media') {
    if $is_controller_media == '0' {
      $tag = 'remote'
    } else {
      $tag = 'local'
    }
  } else {
    $tag = 'remote'
    $is_controller_media = '0'
  }

  if !defined('$is_node_media') {
    $is_node_media = '0'
  }

  if ($is_controller_media == '1' ) and ( $is_node_media == '1' ) {
    $logs=deep_merge($common_logs, deep_merge($controller_logs, $node_logs))
  }
  elsif $is_controller_media == '1' {
    $logs = deep_merge($common_logs, $controller_logs)
  }
  elsif $is_node_media == '1' {
    $logs = deep_merge($common_logs, $node_logs)
  }
  else {
    $logs = $common_logs
  }


  $logs.each |$resource_type,$log| {
    $log.each |$log_name,$values| {
      if $values['loop'] {
        $resource = $values.delete('loop')
        if $loops[$values['loop']] {
          $loops[$values['loop']].each|$loop| {
            if $defaults[$resource_type] {
              $res = deep_merge($defaults[$resource_type], $resource)
            } else {
              $res = $resource
            }
            $config = Hash($res.map |$i, $v| { 
              if $v =~ Hash { 
                [ $i, Hash($v.map |$ii, $vv| { 
                  if $vv =~ Hash {
                    [$ii, Hash($vv.map |$iii,$vvv| {
                      if $vvv =~ Hash {
                        [$iii, Hash($vvv.map |$iiii,$vvvv| {[$iiii, inline_template("<%= \"${vvvv}\" %>")]} )]
                      } elsif $vvv =~ String {
                        [$iii, inline_template("<%= \"${vvv}\" %>")]
                      }
                      else {
                        [$iii, $vvv]
                      }
                    } )]
                  }
                  elsif $vv =~ String {
                    [$ii, inline_template("<%= \"${vv}\" %>")]
                  }
                  else {
                    [$ii, $vv]
                  }
                })]
              } elsif $v =~ String {
                [$i, inline_template("<%= \"${v}\" %>")]
              } else {
                [$i, $v]
              }
            })
            Resource["fluentd::${resource_type}"] { "${log_name}_${loop}":
              * => $config,
              notify => Class['fluentd::service']
            }
          }
        }
      } else {
        if $defaults[$resource_type] {
          $res = deep_merge($defaults[$resource_type], $values)
        } else {
          $res = $values
        }
        $config = Hash($res.map |$i, $v| { 
          if $v =~ Hash { 
            [ $i, Hash($v.map |$ii, $vv| { 
              if $vv =~ Hash {
                [$ii, Hash($vv.map |$iii,$vvv| {
                  if $vvv =~ Hash {
                      [$iii, Hash($vvv.map |$iiii,$vvvv| {[$iiii, inline_template("<%= \"${vvvv}\" %>")]} )]
                    } elsif $vvv =~ String {
                      [$iii, inline_template("<%= \"${vvv}\" %>")]
                    }
                    else {
                      [$iii, $vvv]
                    }
                  } )]
              }
              elsif $vv =~ String {
                [$ii, inline_template("<%= \"${vv}\" %>")]
              }
              else {
                [$ii, $vv]
              }
            })]
          } elsif $v =~ String {
            [$i, inline_template("<%= \"${v}\" %>")]
          } else {
            [$i, $v]
          }
        })
        Resource["fluentd::${resource_type}"] { "${log_name}":
          * => $config,
          notify => Class['fluentd::service']
        }
      }
    }
  }

  if $is_controller_media == '1' {
    file { "/var/lib/ecp-veil/tests/ctr_init_test3.sh":
      ensure  => present,
      content => template("${module_name}/ctr_init_test3.sh.erb"),
      mode => '0550',
    }
  }

  if $is_node_media == '1' {
    file { "/var/lib/ecp-veil/tests/node_init_test3.sh":
      ensure  => present,
      content => template("${module_name}/node_init_test3.sh.erb"),
      mode => '0550',
    }
  }
}
