# configure environment to install node from .deb and dev repo
class space_node(
  Optional[Tuple]   $extra_facts = {},
  Hash[String,Hash] $files       = {},
) {

  require space_utils

  space_facts::addfacter { $name:
    extra_facts => $extra_facts,
  }

  include space_cli
  include space_media_conf
  include veil_global_conf
  include veil_backup_conf

  $veil_global_group = lookup('veil_global_group')
  $veil_node_user = lookup('veil_node_user')

  $file_defaults = {
     owner => $veil_node_user,
     group => $veil_global_group,
     mode  => '0755',
  }

  create_resources(file, $files, $file_defaults)

  # TEST5
  file { "/var/lib/ecp-veil/tests/node_init_test5.sh":
    ensure  => present,
    content => template("${module_name}/node_init_test5.sh.erb"),
    mode => '0550',
  }


  $veil_pgsql_db_user = lookup('veil_pgsql_db_user')
  $veil_pgsql_node_db_name = lookup('veil_pgsql_node_db_name')
  $veil_pgsql_port = lookup('veil_pgsql_port')
  $veil_pgbouncer_port = lookup('veil_pgbouncer_port')
  $pgsql_db_password = lookup('pgsql_db_password')

#  todo:
#  first-init to 'after-post-start' stage
#  require  space_aide

  contain '::space_node::nginx_node_inc'
  contain '::space_node::install'
  contain '::space_node::install_deps'
  contain '::space_node::dirs'
  contain '::space_node::node_conf'
  contain '::space_node::init_services'
  contain '::space_node::first_boot'
  contain '::space_node::kmod'
  contain '::space_node::grub'
  contain '::space_node::finalize'
  contain '::space_node::lvm' ##removed since lvm2-2.03.00
  contain '::space_node::multipath'
  contain '::space_node::journalctl'
  contain '::space_node::o2cb'
  contain '::space_node::dlm'
  contain '::space_node::consul'
  contain '::space_node::minio'
  contain '::space_node::lxc_conf'
  contain '::space_node::nfs_server'

  Class['space_node::nginx_node_inc'] ->
  Class['space_node::node_conf'] ->
  Class['space_node::install'] ->
  Class['space_node::install_deps'] ->
  Class['space_node::dirs'] ->
  Class['space_node::init_services'] ->
  Class['space_node::first_boot'] ->
  Class['space_node::kmod'] ->
  Class['space_node::grub'] ->
  Class['space_node::lvm'] -> ##removed since lvm2-2.03.00
  Class['space_node::finalize'] ->
  Class['space_node::multipath'] ->
  Class['space_node::journalctl'] ->
  Class['space_node::o2cb'] ->
  Class['space_node::dlm'] ->
  Class['space_node::consul'] ->
  Class['space_node::minio'] ->
  Class['space_node::lxc_conf']
  Class['space_node::nfs_server'] ->

  file { "/usr/bin/crsh":
    ensure => absent,
  }


  $net_configured = find_file('/var/lib/ecp-veil/tests/net_configured')
  $veil_env = lookup('veil_env')
  $redis_db_password = lookup('redis_db_password')

  $file_path = '/etc/veil/veil_node_app_secret.key'

  exec { 'gen_veil_node_app_secret':
    command => "/usr/bin/openssl rand -base64 50 > $file_path",
    creates => $file_path,
    path    =>  ["/usr/bin","/usr/sbin", "/bin"],
    onlyif => "test ! -f $file_path"
  }



#  exec { 'node_obfuscate':
#    command => "/var/lib/ecp-veil/node/env/bin/pyarmor obfuscate --exact /var/log/veil/node/settings_local.py --output /var/lib/ecp-veil/node/bazalt --bootstrap 3",
#    onlyif  => "test -f /var/log/veil/node/settings_local.py",
#  }

  if $net_configured {
    exec { "systemctl restart node_supervisor.service":
      subscribe   => File['/var/lib/ecp-veil/node/bazalt/settings_local.py'],
      refreshonly => true,
    }
  }
  if defined('$::is_node_no_controller') {
    if $is_node_no_controller == "1" {

      include space_node::repo_params

      exec {"systemctl restart gunicorn-django-node.service":
        subscribe   => File['/var/lib/ecp-veil/node/bazalt/settings_local.py'],
        refreshonly => true,
      }
#      exec { 'node_cli_obfuscate':
#        command => "/var/lib/ecp-veil/cli/env/bin/pyarmor obfuscate --exact /var/log/veil/node/settings_local.py --output /var/lib/ecp-veil/cli/app",
#        onlyif  => "test -f /var/log/veil/node/settings_local.py",
#      }
    }
  }

  tidy { 'node_tmp_settings_local.py':
    path => '/var/log/veil/node/settings_local.py'
  }
}
