# configure environment to install controller from .deb and dev repo
class space_repos (
  Optional[String] $env   = undef,
  Optional[String] $extra = undef,
  Optional[String] $host  = undef,
){

  $apt_source_base = "/etc/apt/sources.list.d/spacevm.list"
  $apt_source_extra = "/etc/apt/sources.list.d/spacevm-extra.list"

  if $facts['os']['family'] == 'Altlinux' {
    file { "${apt_source_base}":
      content => template("$module_name/spacevm_rpm.list.erb"),
      owner   => 'root',
      group   => 'root',
      mode    => '0644',
    }
  } else {
    file { "/etc/apt/trusted.gpg.d":
      ensure => directory,
    }

    tidy { '/etc/apt/sources.list.d':
      recurse => 1,
      matches => [ 'ecp-veil.list', 'ecp-veil-extra.list' ]
    }

    if $host {
      file { "${apt_source_base}":
        content => template("$module_name/base_repo.list.erb"),
        owner   => 'root',
        group   => 'root',
        mode    => '0644',
        notify  => Exec['apt_update_space']
      }

      if $extra {
        file { "${apt_source_extra}":
          content => template("$module_name/extra_repo.list.erb"),
          owner   => 'root',
          group   => 'root',
          mode    => '0644',
          notify  => Exec['apt_update_space_extra']
        }
      }
    } else {
      file { "${apt_source_base}":
        ensure => absent
      }
      file { "${apt_source_extra}":
        ensure =>  absent
      }
    }

    file { "/etc/apt/trusted.gpg.d/veil-reprepro.gpg":
      source => [ "puppet:///modules/${module_name}/trusted.gpg.d/veil-reprepro.gpg" ],
      owner  => 'root',
      group  => 'root',
      mode   => '0644',
    }

    file { "/etc/apt/trusted.gpg.d/veil-aptly.gpg":
      source => [ "puppet:///modules/${module_name}/trusted.gpg.d/veil-aptly.gpg" ],
      owner  => 'root',
      group  => 'root',
      mode   => '0644',
    }

    file { "/etc/apt/trusted.gpg.d/veil-repo-key.gpg":
      source => [ "puppet:///modules/${module_name}/trusted.gpg.d/veil-repo-key.gpg" ],
      owner  => 'root',
      group  => 'root',
      mode   => '0644',
    }

    exec { "apt_update_space":
      command     => "env DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get update -y -t apt_update_space || true",
      onlyif      => "test -f ${apt_source_base}",
      refreshonly => true
    }
    exec { "apt_update_space_extra":
      command     => "env DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get update -y -t apt_update_space_extra || true",
      onlyif      => "test -f ${apt_source_extra}",
      refreshonly => true
    }
  }
}
