#!/usr/bin/env bash

prepare_env()
{
	pkg install -y rsync rust
	# RUST also needs for py-cryptography
	pip install --upgrade pip && \
	pip install virtualenv==16.7.9 six==1.11.0 pycryptodome==3.9.8 cython==0.29.23 pymdown-extensions==9.4 jinja2==3.0.2 MarkupSafe==2.0.1

	rsync -avz --exclude puppetlabs /var/lib/veil-repo/devops/deb/veil-utils/root/etc/ /etc/
	rsync -avz /var/lib/veil-repo/devops/deb/veil-utils/root/lib/ /lib/
	rsync -avz /var/lib/veil-repo/devops/deb/veil-utils/root/root/ /root/
	rsync -avz /var/lib/veil-repo/devops/deb/veil-utils/root/usr/ /usr/
	rsync -avz /var/lib/veil-repo/devops/deb/veil-utils/root/var/ /var/
	# /var/lib/veil-repo/devops/deb/veil-utils/root/DEBIAN/postinst configure
	cp -a /var/lib/veil-repo/devops/docker/pip.conf /etc/
}


#CLI
prepare_cli()
{
	[ ! -d /var/lib/ecp-veil/cli/app ] && mkdir -p /var/lib/ecp-veil/cli/app
	rsync -avz /var/lib/veil-repo/code/veil-cli-app/ /var/lib/ecp-veil/cli/app/
	cd /var/lib/ecp-veil/cli
	virtualenv --always-copy --python=python3.9 env
	source /var/lib/ecp-veil/cli/env/bin/activate
	cd /var/lib/ecp-veil/cli
	./env/bin/pip install --upgrade --use-deprecated=legacy-resolver pip==23.2.1
	cd /var/lib/veil-repo/code/veil-common/veilrpc
	# skip py-spy
	cp -a requirements.txt requirements.txt-o
	grep -v "py-spy=" requirements.txt-o > requirements.txt
#
# pip install py-spy (veil version - 0.3.12, but latest : v0.3.14)-> maturin:
# building wheels for collected packages: maturin
#    Building wheel for maturin (pyproject.toml): started
#   Building wheel for maturin (pyproject.toml): finished with status 'error'
#   error: subprocess-exited-with-error
# needs cargo:
##                 cmd_obj.run()
#          File "setup.py", line 58, in run
#    raise RuntimeError(
#    RuntimeError: cargo not found in PATH. Please install rust (https://www.rust-lang.org/tools/install) and try again
#[end of output]
# + libclang libs:
	pkg install -y cargo llvm
# py-spy needs to disable/FreeBSD
#
# libvirt.so -> devel/libvirt
# devel/py-libvirt 
#   build deps: pkg-config - 'pkg install pkgconf'
	pkg install -y pkgconf devel/libvirt
	/var/lib/ecp-veil/cli/env/bin/pip3 install -r requirements.txt
#
#
	cp -a /var/lib/veil-repo/code/veil-common/veilrpc/veilrpc /var/lib/ecp-veil/cli/env/lib/python3.9/site-packages/
	cd /var/lib/ecp-veil/cli/app
	cp -a requirements.txt requirements.txt-o
	grep -v veilrpc requirements.txt-o > requirements.txt
	/var/lib/ecp-veil/cli/env/bin/pip3 install -r requirements.txt
#	/var/lib/veil-repo/devops/deb/veil-utils/root/DEBIAN/postinst configure
# ^^ /lib/systemd/system/space-utils-first-boot.service. created
#

mkdir -p /var/log/veil/cli /etc/veil/cli
cat > /etc/veil/cli/config.json<<EOF
{
  "logging": {
    "logDir": "/var/log/veil/cli"
  },
  "redis": {
    "host": "127.0.0.1",
    "password": "bazalt",
    "port": 6379,
    "ssl": {
      "verify_peer": false
    },
    "connectTimeout": 3,
    "connectRetry": 2,
    "database": 0,
    "backoff": {
      "algorithm": "DecorrelatedJitterBackoff",
      "base": 500,
      "cap": 750
    }
  }
}
EOF


	mkdir -p /etc/veil/sentry
	cat > /etc/veil/sentry/cli.json <<EOF
{
  "enabled": true,
  "dsn": "http://e50d3c0de94dc04eaccc5427370f95da@192.168.128.15:9000/18",
  "traces_sample_rate": 1.0,
  "release": "huy001"
}
EOF

	echo "/var/lib/ecp-veil/cli/env/bin/python /var/lib/ecp-veil/cli/app/shell.py"
}

prepare_controller()
{
	# CONTROLLER
	[ ! -d /var/lib/ecp-veil/controller/app ] && mkdir -p /var/lib/ecp-veil/controller/app
	rsync -avz /var/lib/veil-repo/code/veil-controller/ /var/lib/ecp-veil/controller/app/
	cd /var/lib/ecp-veil/controller
	virtualenv --always-copy --python=python3.9 env
	source  /var/lib/ecp-veil/controller/env/bin/activate
	#./env/bin/pip install --upgrade --use-deprecated=legacy-resolver pip==23.2.1
	cd /var/lib/veil-repo/code/veil-common/veilrpc
	#/var/lib/ecp-veil/controller/env/bin/pip3 install -r requirements.txt
	cp -a /var/lib/veil-repo/code/veil-common/veilrpc/veilrpc /var/lib/ecp-veil/controller/env/lib/python3.9/site-packages/
	cd /var/lib/ecp-veil/controller/app
	cp -a requirements.txt requirements.txt-o
	grep -v veilrpc requirements.txt-o > requirements.txt

# psycopg2:
#       Error: pg_config executable not found.
#
#      pg_config is required to build psycopg2
pkg install -y postgresql13-client
# postgresql13-contrib


# ERROR: Failed building wheel for python-ldap
#      In file included from Modules/LDAPObject.c:3:
#      Modules/common.h:15:10: fatal error: 'lber.h' file not found
#      #include <lber.h>
#               ^~~~~~~~
#      1 error generated.
#      error: command '/usr/bin/cc' failed with exit code 1
#      [end of output]
pkg install -y openldap24-client

# 
cp -a /usr/local/include/* /usr/include/
# gssapi ?

# ^^ for kerberos

# redis needs packaging 20.4+
#cp -a requirements.txt requirements.txt-o
#grep -v "packaging=" requirements.txt-o > requirements.txt
#echo "packaging>=20.4" >> requirements.txt

# redis needs packaging 20.4+
#cp -a requirements.txt requirements.txt-o
#grep -v "packaging=" requirements.txt-o > requirements.txt
#echo "packaging>=20.4" >> requirements.txt

# redis needs async-timeout>=4.0.2
# The user requested async-timeout>=4.0.2
# aiohttp 4.0.0a1 depends on async_timeout<4.0 and >=3.0

#cp -a requirements.txt requirements.txt-o
#grep -v "async-timeout=" requirements.txt-o > requirements.txt
#echo "async-timeout>=4.0.2" >> requirements.txt

# why aiohttp on 4.0.0. beta???
# https://github.com/aio-libs/aiohttp/releases
#cp -a requirements.txt requirements.txt-o
#grep -v "aiohttp=" requirements.txt-o > requirements.txt
#echo "aiohttp==3.8.4" >> requirements.txt

#The conflict is caused by:
#The user requested async-timeout>=4.0.2
#aiopg 1.3.1 depends on async-timeout<4.0 and >=3.0
#cp -a requirements.txt requirements.txt-o
#grep -v "aiopg=" requirements.txt-o > requirements.txt
#echo "aiopg==1.4.0" >> requirements.txt
#
/var/lib/ecp-veil/controller/env/bin/pip3 install -r requirements.txt

cp -a /var/lib/veil-repo/devops/deb/veil-controller-app/root/var/lib/ecp-veil/controller/other /var/lib/ecp-veil/controller/
# echo "role: controller" >  /etc/puppetveil/code/environments/production/data/veilrole.yaml
# /root/replace.sh
# /var/lib/veil-repo/devops/deb/veil-controller-app/root/DEBIAN/postinst configure
# /var/lib/ecp-veil/utils/other/first_boot.sh
# warning:
#django.db.utils.IntegrityError: duplicate key value violates unique constraint "auth_user_username_key"
#DETAIL:  Key (username)=(admin) already exists.
#
#Unit veil-first-boot.service does not exist, proceeding anyway.
#Failed to disable unit: Unit file veil-first-boot.service does not exist.
# +
#Warning: This method is deprecated, please use match expressions with Stdlib::Compat::String instead. They are described at https://docs.puppet.com/puppet/latest/reference/lang_data_type.html#match-expressions. at ["/etc/puppetveil/code/environments/production/modules/snmp/manifests/params.pp", 317]:["/etc/puppetveil/code/environments/production/modules/veil_snmpd/manifests/init.pp", 5]
#(location: /etc/puppetveil/code/environments/production/modules/stdlib/lib/puppet/functions/deprecation.rb:36:in `deprecation')
# Warning: This method is deprecated, please use the stdlib validate_legacy function,
# with Stdlib::Compat::Array. There is further documentation for validate_legacy function in the README. at ["/etc/puppetveil/code/environments/production/modules/snmp/manifests/init.pp", 371]:["/etc/puppetveil/code/environments/production/modules/veil_snmpd/manifests/init.pp", 5]
# (location: /etc/puppetveil/code/environments/production/modules/stdlib/lib/puppet/functions/deprecation.rb:36:in `deprecation')
# Warning: Unknown variable: 'snmp_yaml'. (file: /etc/puppetveil/code/environments/production/modules/veil_snmpd/manifests/init.pp, line: 11, column: 23)
# Warning: Arguments to Resource[] are all empty/undefined (file: /etc/puppetveil/code/environments/production/modules/veil_snmpd/manifests/init.pp, line: 11, column: 23)
# No arg for -w option
cat > /etc/veil/sentry/controller.json<<EOF
{
  "enabled": true,
  "dsn": "http://04c8daf74786decba9891a8f7e7afac3@192.168.128.15:9000/19",
  "integrations": true,
  "traces_sample_rate": 1.0
}
EOF

[ ! -d /var/lib/ecp-veil/controller/app/bazalt ] && mkdir -p /var/lib/ecp-veil/controller/app/bazalt
cat > /var/lib/ecp-veil/controller/app/bazalt/settings_local.py<<EOF
import os
SECRET_KEY = "naelahph5ohvoor3chiSiechiilesohyoo9Aekav6oedechaht"
ALLOWED_HOSTS = ["127.0.0.1", "localhost"]
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": "veildb",
        "USER": "veil",
        "PASSWORD": "bazalt",
        "HOST": "127.0.0.1",
        "PORT": "5432",
        'ATOMIC_REQUESTS': True,
        'OPTIONS': {
            'sslmode': 'disable',
        }
    }
}
ISOLATION_LEVEL = 1
LOGS_DIR = "/var/log/veil/controller"
REDIS_PASSWORD = "bazalt"
REDIS_HOST = "localhost"
REDIS_PORT = 6379
REDIS_DB = 0
if REDIS_PASSWORD is not None:
    REDIS_URL = "redis://:%s@%s:%s/%s" % (REDIS_PASSWORD, REDIS_HOST, REDIS_PORT, REDIS_DB)
else:
    REDIS_URL = "redis://%s:%s/%s" % (REDIS_HOST, REDIS_PORT, REDIS_DB)
REDIS_TIMEOUT = 5
CONTROLLER_USER = "veil-controller"
# websockify settings
WS_TOKEN_EXPTIME = 30
WS_PORT = "6080"
# controller settings
CTRL_LOCAL_DATA_PATH = '/var/lib/ecp-veil/controller/local_data/'
# worker user settings
WORKER_USER = 'veil-node'
WORKER_USER_SSH_DIR = "/var/lib/ecp-veil/.users/%s/.ssh/" % WORKER_USER
WORKER_USER_SSH_PRIVATE = os.path.join(WORKER_USER_SSH_DIR, "id_rsa")
WORKER_USER_SSH_PUBLIC = os.path.join(WORKER_USER_SSH_DIR, "id_rsa.pub")
# sync proxy
SYNC_WAIT_TIMEOUT = 60
FORBIDDEN_SYNC_HEADER = "FORBIDDEN_SYNC"
# debug settings
DEBUG = True
OEM = "SpaceVM"
OEM_FULL = "SpaceVM"
OEM_SITE = "https://spacevm.ru"
EOF

}

migrate_db()
{
        local _ret=
	DATE_CMD="date"
        local _st_time=$( ${DATE_CMD} +%s )

	CONTROLLER_ENV_DIR="/var/lib/ecp-veil/controller/env"
	CONTROLLER_APP_DIR="/var/lib/ecp-veil/controller/app"


        ${CONTROLLER_ENV_DIR}/bin/python ${CONTROLLER_APP_DIR}/manage.py migrate 2>&1
        _ret=$?

        if [ ${_ret} -ne 0 ]; then
                echo "* controller migration failed"
	fi
        return ${_ret}
}


prepare_beanstalkd()
{
	pkg install beanstalkd
	sysrc beanstalkd_enable="YES"
	sysrc beanstalkd_flags="-l 0.0.0.0 -p 11300 -z 524280"
# -l 0.0.0.0 -p 11300 -b /var/lib/beanstalkd -z 524280
}

export CRYPTOGRAPHY_OPENSSL_NO_LEGACY=1

#prepare_env
#prepare_cli
# double ? spacevm_controller_client not installed ?
#prepare_cli

prepare_controller
