#!/bin/bash

color_subr="/usr/share/veil/color.subr"
system_subr="/usr/share/veil/system.subr"
deb_subr="/usr/share/veil/deb.subr"

set -e
. ${color_subr}
. ${system_subr}
. ${deb_subr}
set +e


configure_grub()
{
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Configure grub${N0_COLOR}"
	local _need_restart=0

	if replace_different -b /etc/default/grub.bak -s ${UTILS_CONFIG_DIR}/grub -d /etc/default/grub -o root -m 0644; then
		_need_restart=1
	fi
	if [ ${_need_restart} -eq 1 ]; then
		LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Update grub${N0_COLOR}"
		update-grub
	fi
}

configure_timezone()
{
	tz=$( ${CAT_CMD} /etc/timezone )
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} save_timezone ${tz}${N0_COLOR}"

	echo "timezone::timezone: ${tz}" > ${LOCAL_MODULES_DIR}/timezone.yaml
}

collect_netinfo()
{
	IP=$( ${GREP_CMD} address /etc/network/interfaces | ${AWK_CMD} '{print $2}' )
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Ip in collect_netinfo first_boot.sh - ${IP}${N0_COLOR}"
	GW=$( ${GREP_CMD} gateway /etc/network/interfaces | ${AWK_CMD} '{print $2}' )
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Gateway in collect_netinfo first_boot.sh - ${GW}${N0_COLOR}"
	NS=$( ${GREP_CMD} nameserver /etc/network/interfaces | ${AWK_CMD} '{$1=""; print $0}' | ${AWK_CMD} '{for(i=1;i<NF;i++)if(i!=NF){$i=$i","}}1' | ${SED_CMD} 's/ //g' )
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Ns in collect_netinfo first_boot.sh - ${NS}${N0_COLOR}"
	MASK=$( ${GREP_CMD} netmask /etc/network/interfaces | ${AWK_CMD} '{print $2}' )
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Mask in collect_netinfo first_boot.sh - ${MASK}${N0_COLOR}"

	if $( echo ${IP} | ${GREP_CMD} -q '/' ); then
		IS_PREFIX=1
		MASK=
	else
		IS_PREFIX=0
		[ -z "${MASK}" ] && MASK="255.255.255.0"
	fi

	# [ -z "${NS}" ] && NS="8.8.8.8"

	IFACE=$( ${IP_CMD} link | ${GREP_CMD} BROADCAST | ${GREP_CMD} -m 1 "state UP" | ${AWK_CMD} '{printf $2 " "}' | ${TR_CMD} -d : )
	if [ -n "${IFACE}" ]; then
		LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} First UP interface in collect_netinfo first_boot.sh - ${IFACE}${N0_COLOR}"
	else
		IFACE=$( get_default_uplink )
		LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} No UP interface! uplink interface ${IFACE} got by default${N0_COLOR}"
	fi

	VLANID=
	if [ -f "${BASE_DIR}/first_boot_vlan.conf" ]; then
		. ${BASE_DIR}/first_boot_vlan.conf
		${RM_CMD} ${BASE_DIR}/first_boot_vlan.conf
	fi

	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} VLANID in collect_netinfo first_boot.sh - ${VLANID}${N0_COLOR}"
}

migrate_db()
{
	if [ "$controller" -eq 1 ]; then
		echo $( ${DATE_CMD} "+%Y-%m-%d-%H-%M-%S" ) >> ${CONTROLLER_LOGS_DIR}/create_db.log
		${CONTROLLER_ENV_DIR}/bin/python ${CONTROLLER_APP_DIR}/manage.py migrate 2>&1 >> ${CONTROLLER_LOGS_DIR}/create_db.log
		_ret=$?
		if [ "$_ret" -ne 0 ]; then
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} controller migration failed with error code: ${_ret}${N0_COLOR}"
		else
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} controller migration completed successfully${N0_COLOR}"
		fi
	fi

	if [ "$node" -eq 1 ]; then
		echo $( ${DATE_CMD} "+%Y-%m-%d-%H-%M-%S" ) >> ${NODE_LOGS_DIR}/create_db.log
		${NODE_ENV_DIR}/bin/python ${NODE_DIR}/manage.py migrate 2>&1 >> ${NODE_LOGS_DIR}/migrate_node_db.log
		_ret=$?
		if [ "$_ret" -ne 0 ]; then
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} node migration failed with error code: ${_ret}${N0_COLOR}"
		else
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} node migration completed successfully${N0_COLOR}"
		fi
	fi
}

# появилось для сброса в начальное состояние. изначально не было
create_db_user()
{
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE}create_db_user${NORMAL}"

  local retval

  retval=$( sudo -u postgres psql -tc "SELECT 1 FROM pg_catalog.pg_user WHERE usename='${PGSQL_DB_USER}';" 2>/dev/null )

  [ -n "${retval}" ] && return 0

  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE}create db user${NORMAL}"  | tee ${CONTROLLER_LOGS_DIR}/create_db_user.log
  { sudo -u postgres bash 2>&1 | tee -a ${CONTROLLER_LOGS_DIR}/create_db_user.log; } << EOF
psql -c "CREATE ROLE ${PGSQL_DB_USER};"
psql -c "ALTER ROLE ${PGSQL_DB_USER} WITH CREATEDB NOCREATEROLE INHERIT LOGIN ENCRYPTED PASSWORD '${PGSQL_DB_PASSWORD}';"
EOF

  # extra check
  retval=$( sudo -u postgres psql -tc "SELECT 1 FROM pg_catalog.pg_user WHERE usename='${PGSQL_DB_USER}';" 2>/dev/null )

  if [ -z "${retval}" ]; then
    LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE}failed to create role ${PGSQL_DB_USER}${NORMAL}" | tee -a ${CONTROLLER_LOGS_DIR}/create_db_user.log
    exit 1
  fi
}

create_db()
{
	if [ "$controller" -eq 1 ]; then
		# check and create db
		retval=$( sudo -u postgres psql -tc "SELECT 1 FROM pg_catalog.pg_database WHERE datname='${PGSQL_DB_NAME}';" 2>/dev/null )
		if [ ! -n "${retval}" ]; then
			echo "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR}create db${N0_COLOR}" | tee ${CONTROLLER_LOGS_DIR}/create_db.log
			sudo -u postgres psql -c "CREATE DATABASE ${PGSQL_DB_NAME} WITH OWNER='${PGSQL_DB_USER}' LOCALE='en_US.UTF-8' ENCODING='UTF8' TEMPLATE='template0';" 2>&1 | tee -a ${CONTROLLER_LOGS_DIR}/create_db.log
		fi

		# 
		${CONTROLLER_ENV_DIR}/bin/python ${CONTROLLER_APP_DIR}/manage.py migrate

		# Сompile messages
		${CONTROLLER_ENV_DIR}/bin/python ${CONTROLLER_APP_DIR}/manage.py compilemessages  2>&1 >> ${CONTROLLER_LOGS_DIR}/compilemessages.log
		_ret=$?
		if [ "$_ret" -ne 0 ]; then
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} controller compilemessages failed with error code: ${_ret}${N0_COLOR}"
		fi

		# Create first user
		echo "from django.contrib.auth.models import User; User.objects.create_superuser('${DEFAULT_UI_USER}', '${DEFAULT_UI_USER_EMAIL}', '${DEFAULT_UI_PASSWORD}')" | ${CONTROLLER_ENV_DIR}/bin/python ${CONTROLLER_APP_DIR}/manage.py shell 2>&1 >> ${CONTROLLER_LOGS_DIR}/create_db.log
		echo "from django.contrib.auth.models import Group, User; Group.objects.get_or_create(name='Superuser')[0].user_set.add(User.objects.get(username='admin')); Group.objects.get_or_create(name='Administrator')[0].user_set.add(User.objects.get(username='admin')); Group.objects.get_or_create(name='Security Administrator')[0].user_set.add(User.objects.get(username='admin'))" | ${CONTROLLER_ENV_DIR}/bin/python ${CONTROLLER_APP_DIR}/manage.py shell 2>&1 >> ${CONTROLLER_LOGS_DIR}/create_db.log
		
		admin_exist=$( 2>/dev/null env PGCONNECT_TIMEOUT=3 psql -t  postgresql://${PGSQL_DB_USER}:${PGSQL_DB_PASSWORD}@127.0.0.1:5432/${PGSQL_DB_NAME}?sslmode=disable <<EOF
select username from auth_user where username='admin' LIMIT 1;
EOF
	)
		[ ! -n "$admin_exist" ] && { LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} No controller admin user${N0_COLOR}"; return 3; }
	fi

	if [ "$node" -eq 1 ]; then
	  # check and create db
    retval=$( sudo -u postgres psql -tc "SELECT 1 FROM pg_catalog.pg_database WHERE datname='${PGSQL_NODE_DB_NAME}';" 2>/dev/null )
    if [ ! -n "${retval}" ]; then
      echo "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR}create db${N0_COLOR}" | tee ${NODE_LOGS_DIR}/create_db.log
      sudo -u postgres psql -c "CREATE DATABASE ${PGSQL_NODE_DB_NAME} WITH OWNER='${PGSQL_DB_USER}' LOCALE='en_US.UTF-8' ENCODING='UTF8' TEMPLATE='template0';" 2>&1 | tee -a ${NODE_LOGS_DIR}/create_db.log
    fi

	  # Сompile messages
		${NODE_ENV_DIR}/bin/python ${NODE_DIR}/manage.py compilemessages  2>&1 >> ${NODE_LOGS_DIR}/compilemessages.log
		_ret=$?
		if [ "$_ret" -ne 0 ]; then
     	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} node compilemessages failed with error code: ${_ret}${N0_COLOR}"
		fi

		# Create first user
		echo "from django.contrib.auth.models import User; User.objects.create_superuser('${DEFAULT_UI_USER}', '${DEFAULT_UI_USER_EMAIL}', '${DEFAULT_UI_PASSWORD}')" | ${NODE_ENV_DIR}/bin/python ${NODE_DIR}/manage.py shell 2>&1 >> ${NODE_LOGS_DIR}/create_db.log
		echo "from django.contrib.auth.models import Group, User; Group.objects.get_or_create(name='Superuser')[0].user_set.add(User.objects.get(username='admin')); Group.objects.get_or_create(name='Administrator')[0].user_set.add(User.objects.get(username='admin')); Group.objects.get_or_create(name='Security Administrator')[0].user_set.add(User.objects.get(username='admin'))" | ${NODE_ENV_DIR}/bin/python ${NODE_DIR}/manage.py shell 2>&1 >> ${NODE_LOGS_DIR}/create_db.log

		admin_exist=$( 2>/dev/null env PGCONNECT_TIMEOUT=3 psql -t  postgresql://${PGSQL_DB_USER}:${PGSQL_DB_PASSWORD}@127.0.0.1:5432/${PGSQL_NODE_DB_NAME}?sslmode=disable <<EOF
select username from auth_user where username='admin' LIMIT 1;
EOF
	)
		[ ! -n "$admin_exist" ] && { LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} No node admin user${N0_COLOR}"; return 3; }
	fi

}

clear_logs()
{
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Clear logs${N0_COLOR}"
	if [ "$node" -eq 1 ]; then
		[ -f "${NODE_LOGS_DIR}/node.log" ] && > ${NODE_LOGS_DIR}/node.log
	fi
	if [ "$controller" -eq 1 ]; then
		[ -f "${CONTROLLER_LOGS_DIR}/controller.log" ] && > ${CONTROLLER_LOGS_DIR}/controller.log
	fi
}

normal_exit()
{
	[ -d "${BASE_DIR}/tests" ] && touch "${BASE_DIR}/tests/first_boot.passed"
	[ -d "${BASE_DIR}/tests" ] && touch "${BASE_DIR}/tests/net_configured"

	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Exit without reboot${N0_COLOR}"
	exit 0
}

fix_permission()
{
	if [ "$controller" -eq 1 ]; then
		LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Check file permission on controller${N0_COLOR}"

		chown -R ${CONTROLLER_SYS_USER}:${VEIL_GROUP} ${BASE_DIR} 
		chmod 0775 ${BASE_DIR}

		# controller settings
		chown -R ${CONTROLLER_SYS_USER}:${CONTROLLER_SYS_USER} ${CONTROLLER_DIR} ${SYS_USERS_DIR}/${CONTROLLER_SYS_USER} ${CONTROLLER_LOCAL_DATA_DIR}
		chmod 0775 ${CONTROLLER_DIR}
		chmod 0700 ${CONTROLLER_SYS_USER_SSH_CONF_DIR} ${CONTROLLER_LOCAL_DATA_DIR}
	fi

	if [ "$node" -eq 1 ]; then
		LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Check file permission on node${N0_COLOR}"
		chown -R ${NODE_SYS_USER}:${VEIL_GROUP} ${NODE_DIR} ${SYS_USERS_DIR}/${NODE_SYS_USER}
		chmod 0775 ${NODE_DIR}
		chmod 0700 ${NODE_SYS_USER_SSH_CONF_DIR} ${NODE_DIR}/bin ${PPT_NODE_CONFIG_DIR}
		chmod 600 ${NODE_SYS_USER_SSH_CONF_DIR}/config
	fi

}

swap_control() {
	local mem swp
	mem=$( ${CAT_CMD} /proc/meminfo | ${GREP_CMD} MemTotal: | ${AWK_CMD} '{ print $2 }')
	if [ ${mem} -gt $(( ${MEM_SWAP_OFF}*1000 )) ]; then
		LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR}Disabling swap because of enough RAM${N0_COLOR}"
		${SWAPOFF_CMD} -a
		for swp in $( ${SYSTEMCTL_CMD} list-dependencies swap.target | ${GREP_CMD} \.swap$ | ${SED_CMD} 's/^.*\(dev.*\)/\1/'); do
			${SYSTEMCTL_CMD} mask ${swp}
		done
    fi
}

## MAIN
init_color

DEB_LOGFILE="/var/log/veil/first_boot.log"
# if memory (Mbytes) is more than this limit swap is turned off
MEM_SWAP_OFF=16000


SKIP_NET=0
while getopts "s" opt; do
	case "$opt" in
		s) SKIP_NET=1 ;;
	esac
done
shift $((OPTIND - 1))

LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Start first boot ${N0_COLOR}"

if [ "$1" = "nonet" ]; then
	NONET=1
else
	NONET=0
fi

if [ -f /pxe.boot ]; then
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} PXE install ${N0_COLOR}"
	PXE=1
fi

controller=0
node=0

if [ ! -f "$VEIL_MEDIA_CONF" ]; then
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} No such ${VEIL_MEDIA_CONF} ${N0_COLOR}"
	return 1
fi

. "$VEIL_MEDIA_CONF"

configure_grub
configure_timezone

if [ "${controller}" -eq 1 ]; then
	# call puppet apply before DB-related action
	nice -n 19 ionice -c 3 /usr/bin/flock -w0 -x /tmp/pa.lock /root/bin/puppet-apply
fi

create_db_user
create_db
migrate_db

fix_permission
swap_control

# /tmp/nonet created by packer script
# i.e we work on vagrant
if [ -f /tmp/nonet ]; then
  LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} /tmp/nonet exists, skip first boot${N0_COLOR}"
  clear_logs
  normal_exit
fi

# last octet is dynamic from Vagrantfile
if $( ${IP_CMD} a | ${GREP_CMD} 'link/ether' | ${GREP_CMD} -q 00:00:00:00:00: ); then
#	${SYSTEMCTL_CMD} unmask veil-first-boot.service || true
#	${SYSTEMCTL_CMD} disable veil-first-boot.service || true
#	${RM_CMD} -f /lib/systemd/system/veil-first-boot.service || true
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Vagrant box, skip first boot${N0_COLOR}"
	clear_logs
	normal_exit
fi

# openvswitch-switch post install action
${SYSTEMCTL_CMD} status openvswitch-switch.service > /dev/null 2>&1
ret=$?

if [ "$ret" -ne 0 ]; then
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} First_boot error: ${SYSTEMCTL_CMD} status openvswitch-switch.service return $ret code${N0_COLOR}"
	exit ${ret}
fi

LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} openvswitch-switch.service configured successfully${N0_COLOR}"

# post-action veil-first-boot
#${SYSTEMCTL_CMD} unmask veil-first-boot.service || true
#${SYSTEMCTL_CMD} disable veil-first-boot.service || true
#${RM_CMD} -f /lib/systemd/system/veil-first-boot.service || true

LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Cron puppet configuration${N0_COLOR}"

[ ! -d /var/spool/cron/crontabs ] && mkdir -p /var/spool/cron/crontabs
cat > /var/spool/cron/crontabs/root <<EOF
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:/opt/puppetlabs/bin
MAILTO=/dev/null
# Puppet Name: puppetd
1 1 * * * nice -n 19 ionice -c 3 /usr/bin/flock -w0 -x /tmp/pa.lock /root/bin/puppet-apply > ${BASE_LOGS_DIR}/puppet.log 2>&1
# Puppet Name: puppet_boot
@reboot nice -n 19 ionice -c 3 /usr/bin/flock -w0 -x /tmp/pa.lock /root/bin/puppet-apply > ${BASE_LOGS_DIR}/puppet.log 2>&1
EOF

chmod 0600 /var/spool/cron/crontabs/root
chown root:crontab /var/spool/cron/crontabs/root

LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} System synchronization started${N0_COLOR}"

for i in $( ${SEQ_CMD} 1 12 ); do
	nice -n 19 ionice -c 3 /usr/bin/flock -w0 -x /tmp/pa.lock /root/bin/puppet-apply > ${BASE_LOGS_DIR}/puppet.log
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Iteration ${i}, Errcode: $?${N0_COLOR}"
	# controller_node
	if [ ${controller} -eq 1 -a ${node} -eq 1 ]; then
		if [ -f /var/lib/ecp-veil/tests/node_init_finalize.passed ] && [ -f /var/lib/ecp-veil/tests/ctr_init_finalize.passed ]; then
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} All controller_node test passed on loop ${i}${N0_COLOR}"
			break
		else
			passed_tests=$( ${FIND_CMD} /var/lib/ecp-veil/tests/ -name '*.passed' | ${SORT_CMD} )
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} controller_node passed tests on loop ${i}: \n${passed_tests}${N0_COLOR}"
		fi
	# node
	elif [ ${node} -eq 1 ]; then
		if [ -f /var/lib/ecp-veil/tests/node_init_finalize.passed ]; then
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} All node test passed on loop ${i}${N0_COLOR}"
			break
		else
			passed_tests=$( ${FIND_CMD} /var/lib/ecp-veil/tests/ -name '*.passed' | ${SORT_CMD} )
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} node passed tests on loop ${i}: \n${passed_tests}${N0_COLOR}"
		fi
	elif [ ${controller} -eq 1 ]; then
		if [ -f /var/lib/ecp-veil/tests/ctr_init_finalize.passed ]; then
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} All controller test passed on loop ${i}${N0_COLOR}"
			break
		else
			passed_tests=$( ${FIND_CMD} /var/lib/ecp-veil/tests/ -name '*.passed' | ${SORT_CMD} )
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} controller passed tests on loop ${i}: \n${passed_tests}${N0_COLOR}"
		fi
	fi
done

LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} System last state synchronization started${N0_COLOR}"
/usr/local/sbin/veil-role -a full -d > ${BASE_LOGS_DIR}/puppet.log
LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Last synchronization errcode: $?${N0_COLOR}"

LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Network auto-configure started${N0_COLOR}"

IS_DHCP=0
IS_STATIC=0

if egrep -q dhcp /etc/network/interfaces; then
	IS_DHCP=1
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} DHCP network config ${N0_COLOR}"
fi

if egrep -q '([[:digit:]]{1,3}\.){3}[[:digit:]]{1,3}' /etc/network/interfaces; then
	IS_STATIC=1
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Static network config${N0_COLOR}"
fi

if [ "$IS_DHCP" -eq 0 -a "$IS_STATIC" -eq 0 ]; then
	IS_DHCP=1
fi

if [ "$NONET" -eq 1 ]; then
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Nonet, exit${N0_COLOR}"
	normal_exit
fi

collect_netinfo

DT=$( ${DATE_CMD} "+%Y-%m-%d-%H-%M-%S" )

if [ "$SKIP_NET" -eq 1 ]; then
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Skip network configuration${N0_COLOR}"
	${SYSTEMCTL_CMD} restart networking.service
	[ -d "${BASE_DIR}/tests" ] && touch "${BASE_DIR}/tests/net_configured"
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Restart networking errcode: $?${N0_COLOR}"
else
	if [ "$IS_STATIC" -eq 1 ]; then
		LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Static - variable is ${IS_STATIC}${N0_COLOR}"
		if [ -z "$GW" ]; then
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Gateway is empty${N0_COLOR}"
			# exit 1
		else
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Gateway in first_boot.sh in static configuration - ${GW}. Date - ${DT}${N0_COLOR}"
		fi
		if [ -z "$IP" ]; then
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Ip is empty${N0_COLOR}"
			exit 1
		else
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Ip in first_boot.sh in static configuration - ${IP}. Date - ${DT}${N0_COLOR}"
		fi
		LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Ip_prefix in first_boot.sh in static configuration - ${IS_PREFIX}. Date - ${DT}${N0_COLOR}"
		LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Mask in first_boot.sh in static configuration - ${MASK}. Date - ${DT}${N0_COLOR}"
		LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Iface in first_boot.sh in static configuration - ${IFACE}. Date - ${DT}${N0_COLOR}"
		if [ ${IS_PREFIX} -eq 1 ]; then
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} ${UTILS_CONFIG_DIR}/auto_configure_nic.sh -a init -f -m static -d ${NS} -g ${GW} -i ${IP} -u ${IFACE} -t 3 -w ${VLANID}${N0_COLOR}"
			${UTILS_CONFIG_DIR}/auto_configure_nic.sh -a init -f -m static -d ${NS} -g ${GW} -i ${IP} -u ${IFACE} -t 3 -w ${VLANID}
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} auto_configure_nic errcode: $?${N0_COLOR}"
		else
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} ${UTILS_CONFIG_DIR}/auto_configure_nic.sh -a init -f -m static -d ${NS} -g ${GW} -i ${IP} -s ${MASK} -u ${IFACE} -t 3 -w ${VLANID}${N0_COLOR}"
			${UTILS_CONFIG_DIR}/auto_configure_nic.sh -a init -f -m static -d ${NS} -g ${GW} -i ${IP} -s ${MASK} -u ${IFACE} -t 3 -w ${VLANID}
			LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} auto_configure_nic errcode: $?${N0_COLOR}"
		fi
	else
		LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} ${UTILS_CONFIG_DIR}/auto_configure_nic.sh -a init -m dhcp -u ${IFACE} -w ${VLANID}${N0_COLOR}"
		${UTILS_CONFIG_DIR}/auto_configure_nic.sh -a init -m dhcp -u ${IFACE} -w ${VLANID}
	fi
fi

clear_logs

if [ ${controller} -eq 1 -a ${node} -eq 1 ]; then
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Restart controller services${N0_COLOR}"
	restart_services controller-supervisor gunicorn-proxy gunicorn-uploader gunicorn-ws websockify gunicorn-django daphne-django gunicorn-logger gunicorn-apiclient websockify

	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Restart node on controller services${N0_COLOR}"
	restart_services node_supervisor
elif [ ${node} -eq 1 ]; then
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Restart only node services${N0_COLOR}"
	restart_services node_supervisor gunicorn-proxy-node gunicorn-django-node gunicorn-uploader-node
elif [ ${controller} -eq 1 ]; then
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR} Restart only controller services${N0_COLOR}"
	restart_services controller-supervisor gunicorn-proxy gunicorn-uploader gunicorn-ws websockify gunicorn-django daphne-django gunicorn-logger gunicorn-apiclient websockify
fi

${SYSTEMCTL_CMD} is-active --quiet node_supervisor.service && LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR}Node supervisor active${N0_COLOR}" || LOG "* ${H5_COLOR}${MY_APPLICATION}: ${LRED}Node supervisor is not active!${N0_COLOR}"

normal_exit
