#!/bin/sh
color_subr="/usr/share/veil/color.subr"
system_subr="/usr/share/veil/system.subr"
deb_subr="/usr/share/veil/deb.subr"
utils_subr="/usr/share/veil/utils.subr"

set -e
. ${color_subr}
. ${system_subr}
. ${deb_subr}
. ${utils_subr}
set +e

if [ -r /var/lib/ecp-veil/tests/first_boot.passed ]; then
	/var/lib/ecp-veil/utils/other/spacevm-maintenance -m off
	err 0 "already first-booted: /var/lib/ecp-veil/tests/first_boot.passed"
fi

# back-compat
[ -z "${UTILS_CONFIG_DIR}" ] && UTILS_CONFIG_DIR="/var/lib/ecp-veil/utils/other"

# todo: /etc/veil hardcode
backup_conf="/etc/veil/backup.conf"

[ -r ${backup_conf} ] && . ${backup_conf}
[ ! -z "${BACKUP_DIR}" ] && BACKUP_DIR="/var/veil-backup"
[ ! -d "${BACKUP_DIR}" ] && ${MKDIR_CMD} -p ${BACKUP_DIR}

# копирование nginx иерархии на ранней стадии (до начала работы роли), чтобы показать progressbar
# как можно быстрее
nginx_early()
{
	[ -d /etc/nginx ] && ${RM_CMD} -rf /etc/nginx
	${CP_CMD} -a ${UTILS_CONFIG_DIR}/nginx-early/nginx /etc/nginx
	${CP_CMD} -a ${UTILS_CONFIG_DIR}/nginx-early/nginx.maintenance.inc /etc/
	${CP_CMD} -a ${UTILS_CONFIG_DIR}/nginx-early/nginx.slave.inc /etc/
	${CP_CMD} -a ${UTILS_CONFIG_DIR}/nginx-early/veil_repo_params /etc/
	[ -x /usr/sbin/nginx ] && return 0
	${SYSTEMCTL_CMD} enable nginx
	${SYSTEMCTL_CMD} stop nginx
	${SYSTEMCTL_CMD} start nginx
}

create_global_config()
{
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR}create_global_config${N0_COLOR}"

	if [ ! -r ${CFG_ETC_DIR}/global.conf ]; then
		LOG "* ${N4_COLOR}${MY_APPLICATION}: ${H1_COLOR}config file ${CFG_ETC_DIR}/global.conf has been created${N0_COLOR}"
		${CP_CMD} -a ${CFG_ETC_DIR}/global.conf-sample ${CFG_ETC_DIR}/global.conf
	fi
}

create_media_config()
{
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR}create_media_config${N0_COLOR}"

	if [ ! -r ${CFG_ETC_DIR}/media.conf ]; then
		LOG "* ${N4_COLOR}${MY_APPLICATION}: ${H1_COLOR}config file ${CFG_ETC_DIR}/media.conf has been created${N0_COLOR}"
		${CP_CMD} -a ${CFG_ETC_DIR}/media.conf-sample ${CFG_ETC_DIR}/media.conf
	fi
}

config_puppet_conf()
{
	LOG "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR}config_puppet_conf${N0_COLOR}"

	if ! ${GREP_CMD} -q "report = false" /etc/puppetlabs/puppet/puppet.conf; then
		# выключаем писать отчеты в /opt/puppetlabs/puppet/cache/reports
		echo "report = false" >> /etc/puppetlabs/puppet/puppet.conf
	fi
	if ${GREP_CMD} -q "report = true" /etc/puppetlabs/puppet/puppet.conf; then
		${SED_CMD} -i '/report = true/d' /etc/puppetlabs/puppet/puppet.conf
	fi

	if [ ! -d /etc/puppetlabs/puppet/eyaml ]; then
		${MKDIR_CMD} -p /etc/puppetlabs/puppet/eyaml
		${CHMOD_CMD} -R 775 /etc/puppetlabs/puppet/eyaml
	fi
	replace_different -s ${UTILS_CONFIG_DIR}/eyaml_private_key.pkcs7.pem -d /etc/puppetlabs/puppet/eyaml/private_key.pkcs7.pem -m 0775
	replace_different -s ${UTILS_CONFIG_DIR}/eyaml_public_key.pkcs7.pem -d /etc/puppetlabs/puppet/eyaml/public_key.pkcs7.pem -m 0775

	return 0
}

## MAIN

DEB_LOGFILE="/var/log/veil/first_boot.log"
[ ! -d /var/lib/ecp-veil/tests ] && ${MKDIR_CMD} -p /var/lib/ecp-veil/tests
[ ! -r ${DEB_LOGFILE} ] && ${TRUNCATE_CMD} -s0 ${DEB_LOGFILE}
echo "--" >> ${DEB_LOGFILE}
${ECHO} "${N1_COLOR}space-utils first-boot in progress, logs: ${N2_COLOR}/var/log/veil/first_boot.log${N0_COLOR}" | ${TEE_CMD} -a ${DEB_LOGFILE}

# setup maintenance mode
nginx_early | ${TEE_CMD} -a ${DEB_LOGFILE}

PROGRESS=$( ${AWK_CMD} 'BEGIN{srand();print int(rand()*(1-10))+10 }' )
/var/lib/ecp-veil/utils/other/spacevm-maintenance -m on -p ${PROGRESS} -s "SpaceVM first-boot: configure space-utils.."

# reset progressbar at the end of space-utils frst boot?
space_utils_only=1
[ -r /var/lib/ecp-veil/controller/app/requirements.txt -o -r /var/lib/ecp-veil/node/requirements.txt ] && space_utils_only=0

if [ ${space_utils_only} -eq 1 ]; then
	trap "/var/lib/ecp-veil/utils/other/spacevm-maintenance -m off" HUP INT ABRT BUS TERM EXIT
fi

init_color

full_st_time=$( ${DATE_CMD} +%s )

create_global_config
create_media_config
config_puppet_conf

# convert timezone/resolv/root password into YAML
configure_timezone
configure_resolv
configure_root_password

st_time=$( ${DATE_CMD} +%s )
${ECHO} "${N1_COLOR}apply initital roles: ${N2_COLOR}hosts,vaccounts,locales,apt,timezone,systemd,sudo,ssh,limits,sysctl,ntp,dynmotd,crontab,profile::custom_facts,space_repos,space_utils,space_cli,beanstalkd${N0_COLOR}" | ${TEE_CMD} -a ${DEB_LOGFILE}
/usr/local/sbin/veil-role -a hosts,vaccounts,locales,apt,timezone,systemd,sudo,ssh,limits,sysctl,ntp,dynmotd,crontab,profile::custom_facts,space_repos,space_utils,space_cli,beanstalkd | ${TEE_CMD} -a ${DEB_LOGFILE}
_ret=$?
end_time=$( ${DATE_CMD} +%s )

# NTP can make adjustments here, need to check negative val
if [ ${end_time} -gt ${st_time} ]; then
	diff_time=$(( end_time - st_time ))
else
	diff_time=$(( st_time - end_time ))
fi
diff_time=$( displaytime ${diff_time} )
if [ ${_ret} -ne 0 ]; then
	err 1 "${N1_COLOR}apply initial roles failed ${N2_COLOR}in ${diff_time}${N0_COLOR}" | ${TEE_CMD} -a ${DEB_LOGFILE}
else
	${ECHO} "${N1_COLOR}create config done ${N2_COLOR}in ${diff_time}${N0_COLOR}" | ${TEE_CMD} -a ${DEB_LOGFILE}
fi

# todo: IaC
${SYSTEMCTL_CMD} enable veil-watchrole.service > /dev/null || true
${SYSTEMCTL_CMD} stop veil-watchrole.service > /dev/null || true
${SYSTEMCTL_CMD} start veil-watchrole.service > /dev/null || true

[ ! -d /var/spool/cron/crontabs ] && ${MKDIR_CMD} -p /var/spool/cron/crontabs
${CAT_CMD} > /var/spool/cron/crontabs/root <<EOF
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:/opt/puppetlabs/bin
MAILTO=/dev/null
# Puppet Name: puppetd
*/10 * * * * nice -n 19 ionice -c 3 /usr/bin/flock -w0 -x /tmp/pa.lock /root/bin/puppet-apply > ${BASE_LOGS_DIR}/puppet.log 2>&1
# Puppet Name: puppet_boot
@reboot nice -n 19 ionice -c 3 /usr/bin/flock -w0 -x /tmp/pa.lock /root/bin/puppet-apply > ${BASE_LOGS_DIR}/puppet.log 2>&1
EOF

${SYSTEMCTL_CMD} restart cron.service

full_end_time=$( ${DATE_CMD} +%s )
full_diff_time=$(( full_end_time - full_st_time ))
# export to metrics
put_prometheus_file_metrics "first_boot" "first_boot_space_utils" ${full_diff_time}
full_diff_time=$( displaytime ${full_diff_time} )

# Get packages stats
calc_deb_packages_status
# ALL_PACKAGES_NUM and ALL_PACKAGES_BYTES
put_prometheus_file_metrics "first_boot" "first_boot_space_utils_all_packages_num" ${ALL_PACKAGES_NUM}
put_prometheus_file_metrics "first_boot" "first_boot_space_utils_all_packages_bytes" ${ALL_PACKAGES_BYTES}

${ECHO} "${N1_COLOR}space-utils first-boot complete successfull in: ${N2_COLOR}${full_diff_time}${N0_COLOR}" | ${TEE_CMD} -a ${DEB_LOGFILE}

# This file is awaiting a CLI and some postinst scripts of other Space Components
${TOUCH_CMD} /var/lib/ecp-veil/tests/first_boot.passed

exit 0
